@@ -4,20 +4,16 @@ Core implementation of :mod:`sklearndf.regression.wrapper`
 
 import logging
 from abc import ABCMeta
-from typing import Callable, Generic, Optional, Sequence, TypeVar
+from typing import Any, Generic, Optional, TypeVar, Union
 
-import numpy as np
 import pandas as pd
 from sklearn.base import RegressorMixin
 from sklearn.isotonic import IsotonicRegression
 
 from pytools.api import AllTracker
 
-from sklearndf import LearnerDF, RegressorDF
-from sklearndf.transformation.wrapper import (
-    ColumnPreservingTransformerWrapperDF,
-    NumpyTransformerWrapperDF,
-)
+from sklearndf import LearnerDF
+from sklearndf.transformation.wrapper import ColumnPreservingTransformerWrapperDF
 from sklearndf.wrapper import (
     MetaEstimatorWrapperDF,
     RegressorWrapperDF,
@@ -68,13 +64,6 @@ class MetaRegressorWrapperDF(
     pass
 
 
-# noinspection PyProtectedMember
-from ...wrapper._adapter import RegressorNPDF as _RegressorNPDF
-
-# noinspection PyProtectedMember
-from ...wrapper._wrapper import _StackableRegressorDF
-
-
 class StackingRegressorWrapperDF(
     StackingEstimatorWrapperDF[T_NativeRegressor],
     RegressorWrapperDF,
@@ -92,14 +81,6 @@ class StackingRegressorWrapperDF(
 
         return RidgeCVDF()
 
-    def _make_stackable_learner_df(self, learner: LearnerDF) -> _StackableRegressorDF:
-        return _StackableRegressorDF(learner)
-
-    def _make_learner_np_df(
-        self, delegate: RegressorDF, column_names: Callable[[], Sequence[str]]
-    ) -> _RegressorNPDF:
-        return _RegressorNPDF(delegate, column_names)
-
 
 class RegressorTransformerWrapperDF(
     RegressorWrapperDF[T_Regressor],
@@ -115,33 +96,29 @@ class RegressorTransformerWrapperDF(
 
 
 class IsotonicRegressionWrapperDF(
-    RegressorTransformerWrapperDF[IsotonicRegression],
-    NumpyTransformerWrapperDF,
-    metaclass=ABCMeta,
+    RegressorTransformerWrapperDF[IsotonicRegression], metaclass=ABCMeta
 ):
     """
     DF wrapper for :class:`sklearn.isotonic.IsotonicRegression`.
     """
 
     # noinspection PyPep8Naming
-    def _check_parameter_types(
-        self,
-        X: pd.DataFrame,
-        y: Optional[pd.Series],
-        *,
-        expected_columns: pd.Index = None,
-    ) -> None:
-        super()._check_parameter_types(X, y, expected_columns=expected_columns)
+    def _check_parameter_types(self, X: pd.DataFrame, y: Optional[pd.Series]) -> None:
+        super()._check_parameter_types(X=X, y=y)
         if X.shape[1] != 1:
             raise ValueError(
                 f"arg X expected to have exactly 1 column but has {X.shape[1]} columns"
             )
 
     # noinspection PyPep8Naming
-    def _adjust_X_type_for_delegate(
-        self, X: pd.DataFrame, *, to_numpy: Optional[bool] = None
-    ) -> np.ndarray:
-        return super()._adjust_X_type_for_delegate(X).ravel()
+    def _convert_X_for_delegate(self, X: pd.DataFrame) -> Any:
+        return super()._convert_X_for_delegate(X).iloc[:, 0].values
+
+    def _convert_y_for_delegate(
+        self, y: Optional[Union[pd.Series, pd.DataFrame]]
+    ) -> Any:
+        y = super()._convert_y_for_delegate(y)
+        return None if y is None else y.values
 
 
 #
